/****************************************************************************
Copyright (C) 2012 HidraVFX development team
            <https://launchpad.net/~hidravfx-dev-team>

This file is part of the HidraVFX project.

HidraVFX is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HidraVFX is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with HidraVFX. If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/

#include <math.h>
#include <mathplus.h>
#include <vertex.h>

void tpoint_swap(tpoint *a, tpoint *b)
{
  tpoint temp;

  temp.x = a->x;
  temp.y = a->y;
  temp.z = a->z;
  a->x = b->x;
  a->y = b->y;
  a->z = b->z;
  b->x = temp.x;
  b->y = temp.y;
  b->z = temp.z;
}

tpoint tpoint_2d(double xi,double yi)
{
  tpoint p;
  p.x = xi;
  p.y = yi;
  p.z = 0.0;
  return p;
}


tpoint tpoint_3d(double xi,double yi,double zi)
{
  tpoint p;
  p.x = xi;
  p.y = yi;
  p.z = zi;
  return p;
}

tpoint tpoint_copy(tpoint *a)
{
  tpoint p;
  p.x = a->x;
  p.y = a->y;
  p.z = a->z;
  return p;
}

tvertex tvertex_2d(double x0,double y0)
{
  tvertex v;
  v.x = x0;
  v.y = y0;
  v.z = 1.0;
  return v;
}

tvertex tvertex_3d(double x0,double y0,double z0)
{
  tvertex v;
  v.x = x0;
  v.y = y0;
  v.z = z0;
  return v;
}

tvertex tvertex_copy(tvertex *a)
{
  tvertex v;
  v.x = a->x;
  v.y = a->y;
  v.z = a->z;
  return v;
}

tvertex tvertex_null(void)
{
  tvertex v;
  v.x = 0.0;
  v.y = 0.0;
  v.z = 0.0;
  return v;
}

tvertex tvertex_sub(tvertex *v1, tvertex *v2)
{
  tvertex v;
  v.x = v1->x-v2->x;
  v.y = v1->y-v2->y;
  v.z = v1->z-v2->z;
  return v;
}

tvertex tvertex_add(tvertex *v1, tvertex *v2)
{
  tvertex v;
  v.x = v1->x+v2->x;
  v.y = v1->y+v2->y;
  v.z = v1->z+v2->z;
  return v;
}

tvertex tvertex_mul(tvertex *v1, tvertex *v2)
{
  tvertex v;
  v.x = v1->x*v2->x;
  v.y = v1->y*v2->y;
  v.z = v1->z*v2->z;
  return v;
}

double tvertex_length(tvertex *v)
{
  double l;
  l = sqrt(sqr(v->x)+sqr(v->y)+sqr(v->z));
  return l;
}
